#ifndef MKOPCI_H
#define MKOPCI_H

#define MKO_IOC_MAGIC 'X'

#define MAX_MEM_WIN     5
#define MAX_DEVICES_NR 16

#if !defined(__KERNEL__)
#define MaxDeviceCount MAX_DEVICES_NR
typedef struct {
    unsigned short      VendorId;
    unsigned short      DeviceId;
    unsigned short      SubsystemVendorId;
    unsigned short      SubsystemId;
    unsigned int        InstanceId;
    unsigned int        ProcessId;
    unsigned int        LType;
    unsigned short      NumMemWindows;
    unsigned long       MemBase[MAX_MEM_WIN];
    unsigned long       LinBase[MAX_MEM_WIN];
    unsigned int        MemSize[MAX_MEM_WIN];
    unsigned short      IRQ;
    int                 handle;  // device file descriptor
    char                name[sizeof("mkopci00") + 1];
    unsigned char       irq_requested;
    int                 c_bar;
    unsigned char       n_dev;
} mkopcilnx_device_info_t;

typedef struct {
    unsigned short          DeviceCount;
    mkopcilnx_device_info_t DeviceInfo[MaxDeviceCount];
} mkopcilnx_device_table_t;
#else // defined(__KERNEL__)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#endif
struct mkopci_core {
    unsigned short      vendor_id;
    unsigned short      device_id;
    unsigned short      subs_vendor_id;
    unsigned short      subs_id;
    unsigned int        instance;
    unsigned int        process;
    unsigned int        ltype;
    unsigned short      mem_windows_nr;
    unsigned long       mem_base[MAX_MEM_WIN];
    unsigned long       lin_base[MAX_MEM_WIN];
    unsigned int        mem_size[MAX_MEM_WIN];
    unsigned short      irq;
    int                 handle;
    char                name[sizeof("mkopci00") + 1];
    unsigned char       irq_requested;
    int                 c_bar;
    unsigned char       n_dev;
};

// kernel's structure of the driver
struct mkopci_device {
    struct mkopci_core  core;
    struct rw_semaphore rwsem;
    wait_queue_head_t   wq;
    spinlock_t          lock;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
    struct cdev         cdev;
#endif
    unsigned char       omited;
    unsigned int        backdoor;
    struct pci_dev      *pci_dev;
    struct list_head    list;
};
#endif // !defined(_KERNEL__)

#define MKOPCI_IOCTL_GET_VERSION            _IOR(MKO_IOC_MAGIC, 1, int)
#define MKOPCI_IOCTL_GET_BOARDS_COUNT       _IOR(MKO_IOC_MAGIC, 2, int)
#define MKOPCI_IOCTL_GET_DEVICE_TABLE       _IOR(MKO_IOC_MAGIC, 3, struct mkopci_core)
#define MKOPCI_IOCTL_ATTACH_IRQ             _IO(MKO_IOC_MAGIC,  4)
#define MKOPCI_IOCTL_WAIT_IRQ               _IO(MKO_IOC_MAGIC,  5)
#define MKOPCI_IOCTL_DETACH_IRQ             _IO(MKO_IOC_MAGIC,  6)
#define MKOPCI_IOCTL_REQUEST_BAR            _IOW(MKO_IOC_MAGIC, 7, int)
#define MKOPCI_IOCTL_CWPID                  _IO(MKO_IOC_MAGIC,  8)

#define MKO_IOC_MAXNR 8

#endif

